@@ -1,6 +1,5 @@
-# -*- coding: utf-8 -*-
-
 # Copyright 2015 Spanish National Research Council
+# Copyright 2015 LIP - INDIGO-DataCloud
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -96,6 +95,12 @@ volumes = {
             "size": 5,
             "status": "in-use",
         },
+        {
+            "id": uuid.uuid4().hex,
+            "displayName": "volume-nodev",
+            "size": 6,
+            "status": "in-use",
+        },
     ],
 }
 
@@ -120,8 +125,14 @@ pools = {
 }
 
 linked_vm_id = uuid.uuid4().hex
+linked_vm_id_2 = uuid.uuid4().hex
 
-allocated_ip = "192.168.253.23"
+allocated_ip = {"ip": "192.168.253.23",
+                "id": 1,
+                "pool": uuid.uuid4().hex,
+                "instance_id": None}
+
+action_loc_id = uuid.uuid4().hex
 
 floating_ips = {
     tenants["foo"]["id"]: [],
@@ -144,6 +155,53 @@ floating_ips = {
     ],
 }
 
+networks = {
+    tenants["foo"]["id"]: [],
+    tenants["bar"]["id"]: [],
+    tenants["baz"]["id"]: [
+        {"id": uuid.uuid4().hex},
+        {"id": uuid.uuid4().hex}
+        ]
+}
+
+ports = {
+    tenants["foo"]["id"]: [
+        {
+            "port_id": uuid.uuid4().hex,
+            "fixed_ips":
+                [{"ip_address": uuid.uuid4().hex}],
+            "mac_addr": uuid.uuid4().hex,
+            "port_state": "DOWN",
+            "net_id": uuid.uuid4().hex,
+            "server_id": linked_vm_id
+        },
+    ],
+    tenants["bar"]["id"]: [],
+    tenants["baz"]["id"]: [
+        {
+            "port_id": uuid.uuid4().hex,
+            "fixed_ips": [
+                {"ip_address": "192.168.253.1"}
+            ],
+            "mac_addr": uuid.uuid4().hex,
+            "port_state": "ACTIVE",
+            "net_id": uuid.uuid4().hex,
+            "server_id": linked_vm_id
+        },
+        {
+            "port_id": uuid.uuid4().hex,
+            "fixed_ips": [
+                {"ip_address": "192.168.253.2"}
+            ],
+            "mac_addr": uuid.uuid4().hex,
+            "port_state": "ACTIVE",
+            "net_id": uuid.uuid4().hex,
+            "server_id": linked_vm_id_2
+        },
+
+    ],
+}
+
 servers = {
     tenants["foo"]["id"]: [
         {
@@ -152,6 +210,10 @@ servers = {
             "flavor": {"id": flavors[1]["id"]},
             "image": {"id": images["foo"]["id"]},
             "status": "ACTIVE",
+            "security_groups":[
+                {"name": "group1"},
+                {"name": "group2"}
+            ]
         },
         {
             "id": uuid.uuid4().hex,
@@ -159,6 +221,9 @@ servers = {
             "flavor": {"id": flavors[2]["id"]},
             "image": {"id": images["bar"]["id"]},
             "status": "SHUTOFF",
+            "security_groups":[
+                {"name": "group1"}
+            ]
         },
         {
             "id": uuid.uuid4().hex,
@@ -166,6 +231,9 @@ servers = {
             "flavor": {"id": flavors[1]["id"]},
             "image": {"id": images["bar"]["id"]},
             "status": "ERROR",
+            "security_groups":[
+                {"name": "group2"}
+            ]
         },
     ],
     tenants["bar"]["id"]: [],
@@ -176,14 +244,50 @@ servers = {
             "flavor": {"id": flavors[1]["id"]},
             "image": {"id": images["bar"]["id"]},
             "status": "ACTIVE",
+            "os-extended-volumes:volumes_attached": [
+                {"id": volumes[tenants["baz"]["id"]][0]["id"]},
+                {"id": volumes[tenants["baz"]["id"]][1]["id"]}
+            ],
+            "addresses": {
+                "private": [
+                    {"addr": (
+                        (ports[tenants["baz"]["id"]]
+                         [0]["fixed_ips"][0]["ip_address"])
+                    ),
+                        "OS-EXT-IPS:type": "fixed",
+                        "OS-EXT-IPS-MAC:mac_addr": (
+                            ports[tenants["baz"]["id"]][0]["mac_addr"]
+                        )
+                    },
+                    {"addr": floating_ips[tenants["baz"]["id"]][0]["ip"],
+                     "OS-EXT-IPS:type": "floating",
+                     "OS-EXT-IPS-MAC:mac_addr": "1234"},
+                ]
+            },
+            "security_groups":[
+                {"name": "group1"}
+            ]
+        },
+        {
+            "id": linked_vm_id_2,
+            "name": "withvolume",
+            "flavor": {"id": flavors[1]["id"]},
+            "image": {"id": images["bar"]["id"]},
+            "status": "ACTIVE",
             "os-extended-volumes:volumes_attached": [
                 {"id": volumes[tenants["baz"]["id"]][0]["id"]}
             ],
             "addresses": {
                 "private": [
-                    {"addr": floating_ips[tenants["baz"]["id"]][0]["fixed_ip"],
-                     "OS-EXT-IPS:type": "fixed",
-                     "OS-EXT-IPS-MAC:mac_addr": "1234"},
+                    {"addr": (
+                        (ports[tenants["baz"]["id"]]
+                         [1]["fixed_ips"][0]["ip_address"])
+                    ),
+                        "OS-EXT-IPS:type": "fixed",
+                        "OS-EXT-IPS-MAC:mac_addr": (
+                            ports[tenants["baz"]["id"]][1]["mac_addr"]
+                        )
+                    },
                     {"addr": floating_ips[tenants["baz"]["id"]][0]["ip"],
                      "OS-EXT-IPS:type": "floating",
                      "OS-EXT-IPS-MAC:mac_addr": "1234"},
@@ -207,6 +311,63 @@ volumes[tenants["baz"]["id"]][0]["attachments"] = [{
     "id": volumes[tenants["baz"]["id"]][0]["id"],
 }]
 
+volumes[tenants["baz"]["id"]][1]["attachments"] = [{
+    "server_id": servers[tenants["baz"]["id"]][0]["id"],
+    "serverId": servers[tenants["baz"]["id"]][0]["id"],
+    "attachment_id": uuid.uuid4().hex,
+    "volumeId": volumes[tenants["baz"]["id"]][1]["id"],
+    "volume_id": volumes[tenants["baz"]["id"]][1]["id"],
+    "id": volumes[tenants["baz"]["id"]][0]["id"],
+}]
+
+security_groups = {
+    tenants["foo"]["id"]: [],
+    tenants["baz"]["id"]: [
+        {
+            "name": "group1",
+            "id": uuid.uuid4().hex,
+            "description": "group one",
+            "rules": [
+                {"from_port": 443,
+                 "to_port": 443, "ip_range": {"cidr": "10.0.0.0/32"},
+                 "ip_protocol": "tcp"},
+                {"from_port": "1000",
+                 "to_port": 2000, "ip_range": {"cidr": "11.0.0.0/32"},
+                 "ip_protocol": "udp"},
+            ]
+        },
+        {
+            "name": "group2",
+            "id": uuid.uuid4().hex,
+            "description": "group two",
+            "rules": [
+                {"from_port": 80,
+                 "to_port": 80, "ip_range": {"cidr": "10.0.0.0/32"},
+                 "ip_protocol": "tcp"},
+                {"from_port": "4000",
+                 "to_port": 7000, "ip_range": {"cidr": "13.0.0.0/32"},
+                 "ip_protocol": "udp"},
+            ]
+        }
+
+    ],
+    tenants["bar"]["id"]: [
+        {
+            "name": "group3",
+            "id": uuid.uuid4().hex,
+            "description": "group three",
+            "rules": [
+                {"from_port": 443,
+                 "to_port": 443, "ip_range": {"cidr": "10.0.0.0/32"},
+                 "ip_protocol": "tcp"},
+                {"from_port": "1000",
+                 "to_port": 2000, "ip_range": {"cidr": "11.0.0.0/32"},
+                 "ip_protocol": "udp"},
+            ]
+        },
+    ]
+}
+
 
 def fake_query_results():
     cats = []
@@ -251,6 +412,10 @@ def fake_query_results():
         'suspend; '
         'scheme="http://schemas.ogf.org/occi/infrastructure/compute/action#"; '
         'class="action"; title="suspend compute instance"')
+    cats.append(
+        'save; '
+        'scheme="http://schemas.ogf.org/occi/infrastructure/compute/action#"; '
+        'class="action"; title="save compute instance"')
 
     # OCCI Templates
     cats.append(
@@ -302,7 +467,8 @@ def fake_query_results():
     cats.append(
         'ipnetwork; '
         'scheme="http://schemas.ogf.org/occi/infrastructure/network#"; '
-        'class="mixin"; title="IP Networking Mixin"')
+        'class="mixin"; title="IP Networking Mixin"; '
+        'location="%s/ipnetwork/"' % application_url)
     cats.append(
         'up; '
         'scheme="http://schemas.ogf.org/occi/infrastructure/network/action#"; '
@@ -321,7 +487,14 @@ def fake_query_results():
         'ipnetworkinterface; '
         'scheme="http://schemas.ogf.org/occi/infrastructure/'
         'networkinterface#"; '
-        'class="mixin"; title="IP Network interface Mixin"')
+        'class="mixin"; title="IP Network interface Mixin"; '
+        'location="%s/ipnetworkinterface/"' % application_url)
+    cats.append(
+        'ipreservation; '
+        'scheme="http://schemas.ogf.org/occi/infrastructure#"; '
+        'class="kind"; title="IPReservation"; '
+        'rel="http://schemas.ogf.org/occi/infrastructure#network"; '
+        'location="%s/ipreservation/"' % application_url)
 
     # OCCI Infrastructure Storage
     cats.append(
@@ -361,11 +534,25 @@ def fake_query_results():
     cats.append(
         'user_data; '
         'scheme="http://schemas.openstack.org/compute/instance#"; '
-        'class="mixin"; title="Contextualization extension - user_data"')
+        'class="mixin"; title="Contextualization extension - user_data"; '
+        'location="%s/openstack_user_data/"' % application_url)
     cats.append(
         'public_key; '
         'scheme="http://schemas.openstack.org/instance/credentials#"; '
-        'class="mixin"; title="Contextualization extension - public_key"')
+        'class="mixin"; title="Contextualization extension - public_key"; '
+        'location="%s/openstack_public_key/"' % application_url)
+
+    # OCCI contextualization
+    cats.append(
+        'user_data; '
+        'scheme="http://schemas.ogf.org/occi/infrastructure/compute#"; '
+        'class="mixin"; title="Contextualization mixin"; '
+        'location="%s/user_data/"' % application_url)
+    cats.append(
+        'ssh_key; '
+        'scheme="http://schemas.ogf.org/occi/infrastructure/credentials#"; '
+        'class="mixin"; title="Credentials mixin"; '
+        'location="%s/ssh_key/"' % application_url)
 
     result = []
     for c in cats:
@@ -417,6 +604,17 @@ class FakeApp(object):
                            "os-floating-ip-pools")
             self._populate(path, "floating_ip", floating_ips[tenant["id"]],
                            "os-floating-ips")
+            self._populate_server_links(path, "os-interface",
+                                        "interfaceAttachments",
+                                        servers[tenant["id"]],
+                                        ports[tenant["id"]])
+            self._populate_server_links(path, "os-security-groups",
+                                        "security_groups",
+                                        servers[tenant["id"]],
+                                        security_groups[tenant["id"]])
+            self._populate(path, "security_group",
+                           security_groups[tenant["id"]],
+                           "os-security-groups")
             # NOTE(aloga): dict_values un Py3 is not serializable in JSON
             self._populate(path, "image", list(images.values()))
             self._populate(path, "flavor", list(flavors.values()))
@@ -441,7 +639,9 @@ class FakeApp(object):
 
             if actions:
                 action_path = "%s/action" % obj_path
-                self.routes[action_path] = webob.Response(status=202)
+                r = webob.Response(status=202)
+                r.headers["Location"] = action_loc_id
+                self.routes[action_path] = r
 
     def _populate_attached_volumes(self, path, server_list, vol_list):
         for s in server_list:
@@ -460,6 +660,35 @@ class FakeApp(object):
                 self.routes[obj_path] = create_fake_json_resp(
                     {"volumeAttachment": attach})
 
+    def _populate_server_links(self, path, resource, obj,
+                               servers_list, link_list):
+        if servers_list:
+            for s in servers_list:
+                list_obj = []
+                path_base = "%s/servers/%s/%s" % (
+                    path,
+                    s["id"],
+                    resource
+                )
+                for l in link_list:
+                    list_obj.append(l)
+                self.routes[path_base] = create_fake_json_resp(
+                    {obj: list_obj})
+
+    def _populate_ports(self, path, servers_list, ports_list):
+        for s in servers_list:
+            p_list = []
+            path_base = "%s/servers/%s/%s" % (
+                path,
+                s["id"],
+                "os-interface"
+            )
+            for p in ports_list:
+                if p["server_id"] == s["id"]:
+                    p_list.append(p)
+            self.routes[path_base] = create_fake_json_resp(
+                {"interfaceAttachments": p_list})
+
     @webob.dec.wsgify()
     def __call__(self, req):
         if req.method == "GET":
@@ -488,6 +717,26 @@ class FakeApp(object):
                         "status": "on-line"}}
         return create_fake_json_resp(s)
 
+    def _do_resize_server(self, req):
+        body = req.json_body.copy()
+        new_flavor = body.popitem()[1]["flavorRef"]
+        r = self._get_from_routes(req)
+        # Make sure subsequent calls to the server are up to date
+        # Some request path knowledge used which should be in this form:
+        # /v2.1/tenant_id/servers/server_id/action
+        p = req.path.split("/")
+        tenant_id = p[2]
+        server_id = p[4]
+        for s in servers[tenant_id]:
+            if s["id"] == server_id:
+                # do a conversion to avoid current id schemas
+                s["flavor"]["id"] = int(new_flavor)
+                break
+        # And repopulate the objects so they are properly returned
+        self._populate("/%s" % tenant_id, "server", servers[tenant_id],
+                       actions=True)
+        return r
+
     def _do_create_attachment(self, req):
         v = {"volumeAttachment": {"serverId": "foo",
                                   "volumeId": "bar",
@@ -505,9 +754,29 @@ class FakeApp(object):
             else:
                 exc = webob.exc.HTTPNotFound()
                 return FakeOpenStackFault(exc)
-        ip = {"floating_ip": {"ip": allocated_ip, "id": 1}}
+        ip = {"floating_ip": allocated_ip}
         return create_fake_json_resp(ip, 202)
 
+    def _do_create_port(self, req):
+        req_content = req.path_info.split('/')
+        tenant = req_content[1]
+        server = req_content[3]
+        body = req.json_body.copy()
+        net = body["interfaceAttachment"]["net_id"]
+        port = ports[tenant]
+        p = {"interfaceAttachment": {
+            "port_id": uuid.uuid4().hex,
+            "fixed_ips":
+                [{
+                    "ip_address": port[0]["fixed_ips"][0]["ip_address"]
+                }],
+            "mac_addr": port[0]["mac_addr"],
+            "port_state": "DOWN",
+            "net_id": net,
+            "server_id": server
+        }}
+        return create_fake_json_resp(p, 200)
+
     def _do_post(self, req):
         if req.path_info.endswith("servers"):
             return self._do_create_server(req)
@@ -516,13 +785,19 @@ class FakeApp(object):
         elif req.path_info.endswith("action"):
             body = req.json_body.copy()
             action = body.popitem()
-            if action[0] in ["os-start", "os-stop", "reboot",
-                             "addFloatingIp", "removeFloatingIp"]:
+            if action[0] in ["os-start", "os-stop", "reboot", "createImage",
+                             "addFloatingIp", "removeFloatingIp",
+                             "removeSecurityGroup",
+                             "addSecurityGroup"]:
                 return self._get_from_routes(req)
+            if action[0] == "resize":
+                return self._do_resize_server(req)
         elif req.path_info.endswith("os-volume_attachments"):
             return self._do_create_attachment(req)
         elif req.path_info.endswith("os-floating-ips"):
             return self._do_allocate_ip(req)
+        elif req.path_info.endswith("os-interface"):
+            return self._do_create_port(req)
         raise Exception
 
     def _do_delete(self, req):
@@ -531,7 +806,8 @@ class FakeApp(object):
             r"/[^/]+/servers/[^/]+/os-volume_attachments/[^/]+$": 202,
             r"/[^/]+/os-floating-ips/[^/]+$": 202,
             r"/[^/]+/servers/[^/]+$": 204,
-            r"/[^/]+/os-volumes/[^/]+$": 204,
+            r"/[^/]+/os-volumes/[^/]+$": 202,
+            r"/[^/]+/servers/[^/]+/os-interface/[^/]+$": 204,
         }
         for p, st in tested_paths.items():
             if re.match(p, req.path_info):
