@@ -1,6 +1,5 @@
-# -*- coding: utf-8 -*-
-
 # Copyright 2015 Spanish National Research Council
+# Copyright 2015 LIP - INDIGO-DataCloud
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -21,6 +20,7 @@ import routes.middleware
 import webob.dec
 
 import ooi.api.compute
+import ooi.api.ip_reservation
 import ooi.api.network
 import ooi.api.network_link
 from ooi.api import query
@@ -38,17 +38,28 @@ LOG = logging.getLogger(__name__)
 
 occi_opts = [
     config.cfg.StrOpt('ooi_listen',
-                      default="0.0.0.0",
+                      default="0.0.0.0",  # nosec
                       help='The IP address on which the OCCI (ooi) API '
-                      'will listen.'),
+                           'will listen.'),
     config.cfg.IntOpt('ooi_listen_port',
                       default=8787,
                       help='The port on which the OCCI (ooi) API '
-                      'will listen.'),
+                           'will listen.'),
     config.cfg.IntOpt('ooi_workers',
                       help='Number of workers for OCCI (ooi) API service. '
-                      'The default will be equal to the number of CPUs '
-                      'available.'),
+                           'The default will be equal to the number of CPUs '
+                           'available.'),
+    config.cfg.StrOpt('ooi_secure_proxy_ssl_header',
+                      default=None,
+                      help='The HTTP header used to determine the scheme '
+                           'for the original request, even if it was '
+                           'removed by an SSL terminating proxy. Typical '
+                           'value is "HTTP_X_FORWARDED_PROTO".'),
+    # NEUTRON
+    config.cfg.StrOpt('neutron_ooi_endpoint',
+                      default=None,
+                      help='Neutron end point which access to'
+                           ' the Neutron Restful API.'),
 ]
 
 CONF = config.cfg.CONF
@@ -56,6 +67,13 @@ CONF.register_opts(occi_opts)
 
 
 class Request(webob.Request):
+    def __init__(self, environ, *args, **kwargs):
+        if CONF.ooi_secure_proxy_ssl_header:
+            scheme = environ.get(CONF.ooi_secure_proxy_ssl_header)
+            if scheme:
+                environ['wsgi.url_scheme'] = scheme
+        super(Request, self).__init__(environ, *args, **kwargs)
+
     def should_have_body(self):
         return self.method in ("POST", "PUT")
 
@@ -92,8 +110,21 @@ class Request(webob.Request):
 
 class OCCIMiddleware(object):
 
-    occi_version = "1.1"
-    occi_string = "OCCI/%s" % occi_version
+    occi_version = "1.2"
+    supported_occi_versions = ["1.1", occi_version]
+
+    _occi_string = "OCCI/%(occi_version)s"
+
+    @property
+    def occi_string(self):
+        """Return the default OCCI supported version string."""
+        return self._occi_string % {"occi_version": self.occi_version}
+
+    @property
+    def supported_occi_strings(self):
+        """Return all the OCCI supported version strings."""
+        return [self._occi_string % {"occi_version": occi_version}
+                for occi_version in self.supported_occi_versions]
 
     @classmethod
     def factory(cls, global_conf, **local_conf):
@@ -104,17 +135,42 @@ class OCCIMiddleware(object):
             return cls(app, **local_conf)
         return _factory
 
-    def __init__(self, application, openstack_version="/v2.1"):
+    def __init__(self, application, openstack_version="/v2.1",
+                 neutron_ooi_endpoint=None):
         self.application = application
         self.openstack_version = openstack_version
 
-        self.resources = {}
+        if CONF.neutron_ooi_endpoint:
+            self.neutron_ooi_endpoint = CONF.neutron_ooi_endpoint
+        elif neutron_ooi_endpoint:
+            exception.raise_deprecation_message(
+                "Configuration of neutron_ooi_endpoint"
+                " in api-paste.ini file is deprecated,"
+                " include it in nova.conf")
+            self.neutron_ooi_endpoint = neutron_ooi_endpoint
+        else:
+            self.neutron_ooi_endpoint = None
 
+        self.resources = {}
         self.mapper = routes.Mapper()
         self._setup_routes()
 
-    def _create_resource(self, controller):
-        return Resource(controller(self.application, self.openstack_version))
+    def _create_resource(self, controller, neutron_ooi_endpoint=None):
+        if neutron_ooi_endpoint:
+            return Resource(controller(self.application,
+                                       self.openstack_version,
+                                       neutron_ooi_endpoint))
+        return Resource(controller(self.application,
+                                   self.openstack_version))
+
+    def _create_resource_network(self, controller, neutron_ooi_endpoint=None):
+        if neutron_ooi_endpoint:
+            return Resource(controller(
+                neutron_ooi_endpoint=neutron_ooi_endpoint)
+            )
+        else:
+            return Resource(controller(self.application,
+                                       self.openstack_version))
 
     def _setup_resource_routes(self, resource, controller):
         path = "/" + resource
@@ -180,7 +236,10 @@ class OCCIMiddleware(object):
         """
         self.mapper.redirect("", "/")
 
-        self.resources["query"] = self._create_resource(query.Controller)
+        self.resources["query"] = self._create_resource(
+            query.Controller,
+            self.neutron_ooi_endpoint
+        )
         self.mapper.connect("query", "/-/",
                             controller=self.resources["query"],
                             action="index")
@@ -207,11 +266,16 @@ class OCCIMiddleware(object):
         self._setup_resource_routes("networklink",
                                     self.resources["networklink"])
 
-        self.resources["network"] = self._create_resource(
-            ooi.api.network.Controller)
+        self.resources["network"] = self._create_resource_network(
+            ooi.api.network.Controller, self.neutron_ooi_endpoint)
         self._setup_resource_routes("network",
                                     self.resources["network"])
 
+        self.resources["ipreservation"] = self._create_resource(
+            ooi.api.ip_reservation.Controller)
+        self._setup_resource_routes("ipreservation",
+                                    self.resources["ipreservation"])
+
     @webob.dec.wsgify(RequestClass=Request)
     def __call__(self, req):
         response = self.process_request(req)
@@ -225,9 +289,10 @@ class OCCIMiddleware(object):
             # FIXME(aloga): review the regexp, since it will only match the
             # first string
             match = re.search(r"\bOCCI/\d\.\d\b", req.user_agent)
-            if match and self.occi_string != match.group():
+            if match and not any([i == match.group()
+                                 for i in self.supported_occi_strings]):
                 return Fault(webob.exc.HTTPNotImplemented(
-                             explanation="%s not supported" % match.group()))
+                    explanation="%s not supported" % match.group()))
 
         match = self.mapper.match(req.path_info, req.environ)
         if not match:
@@ -238,7 +303,7 @@ class OCCIMiddleware(object):
     def process_response(self, response):
         """Process a response by adding our headers."""
         server_string = "ooi/%s %s" % (version.version_string,
-                                       self.occi_string)
+                                       " ".join(self.supported_occi_strings))
 
         headers = (("server", server_string),)
         if isinstance(response, Fault):
@@ -432,12 +497,12 @@ class ResourceExceptionHandler(object):
 
         if isinstance(ex_value, exception.OCCIException):
             raise Fault(exception.ConvertedException(
-                        code=ex_value.code,
-                        explanation=ex_value.format_message()))
+                code=ex_value.code,
+                explanation=ex_value.format_message()))
         elif isinstance(ex_value, exception.NotImplemented):
             raise Fault(exception.ConvertedException(
-                        code=ex_value.code,
-                        explanation=ex_value.format_message()))
+                code=ex_value.code,
+                explanation=ex_value.format_message()))
         elif isinstance(ex_value, TypeError):
             exc_info = (ex_type, ex_value, ex_traceback)
             LOG.error('Exception handling resource: %s', ex_value,
